/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2017 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.38                                             *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : OS_CSemaphore.c
Purpose : embOS sample program demonstrating the usage of counting
          semaphores.
          Suppose a library has 3 identical study rooms, to be used by
          one student at a time. Students must request a room from the
          front desk if they wish to use a study room. If no rooms are
          free, students wait at the desk until someone relinquishes a
          room. When a student has finished using a room, the student
          must return to the desk and indicate that one room has become
          free.
*/

#include "RTOS.h"
#include <stdio.h>

#define NUM_OF_ROOMS     3
#define NUM_OF_STUDENTS  5

static OS_STACKPTR int Stack[NUM_OF_STUDENTS][128];  // Task stack
static OS_TASK         TCB[NUM_OF_STUDENTS];         // Task-control-block
static OS_CSEMA        CSema;                        // Counting semaphore

static void Student(void* pData) {
  unsigned int StudentNo;
  StudentNo = (unsigned int)pData;
  while(1) {
    OS_WaitCSema(&CSema);    // Get a key for a room and wait at the desk if no room is free
    printf("Student %u enters room.\n", StudentNo);
    OS_Delay(100);           // Occupy room for while   
    printf("Student %u leaves room.\n", StudentNo);
    OS_SignalCSema(&CSema);  // Bring the key back to the desk
    OS_Yield();              // Go to canteen... 
  }
}

/*********************************************************************
*
*       main()
*/
int main(void) {
  OS_InitKern();  // Initialize embOS
  OS_InitHW();    // Initialize required hardware
  OS_CREATETASK_EX(&TCB[0], "Student 1", Student, 100, Stack[0], (void*)1);
  OS_CREATETASK_EX(&TCB[1], "Student 2", Student, 100, Stack[1], (void*)2);
  OS_CREATETASK_EX(&TCB[2], "Student 3", Student, 100, Stack[2], (void*)3);
  OS_CREATETASK_EX(&TCB[3], "Student 4", Student, 100, Stack[3], (void*)4);
  OS_CREATETASK_EX(&TCB[4], "Student 5", Student, 100, Stack[4], (void*)5);
  OS_CreateCSema(&CSema, NUM_OF_ROOMS);
  OS_Start();     // Start embOS
  return 0;
}

/*************************** End of file ****************************/
