/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2019 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.38                                             *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : Startup code for the OS and RISC-V core
*/

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#ifndef   USE_ROM_VECTOR_TABLE
  #define USE_ROM_VECTOR_TABLE  (1)
#endif

#define MSTATUS_MPP 0x00001800

#define NUM_CALLER_SAVE_REGISTERS (1+3+8+4) // save caller integer registers ra, t0-t2, a0-a7, t3-t6

/*********************************************************************
*
*     Publics
*
**********************************************************************
*/

        .global _start
        .type   _start, function

        .global start
        .type   start, function

        .global exit
        .type   exit, function

/*********************************************************************
*
*     Externals, code
*
**********************************************************************
*/

        .extern OS_IRQ_Handler  // Implemented in RTOSInit.c

/*********************************************************************
*
*     CODE segment
*
**********************************************************************
*/

  .section .init, "ax", %progbits
_start:
  la gp, __sdata_start__+0x800
  la sp, __stack_end__
  la tp, __tbss_start__
#if (USE_ROM_VECTOR_TABLE == 0)
  la a0, trap_entry
#else
  la a0, vtrap_entry
  ori a0, a0, 0x1  // Use vectored mode
#endif
  csrw mtvec, a0
  csrw mcause, x0

  /* Call _init */
  la t1, _init
  jalr t1

  /* Load data section */
  la a0, __data_load_start__
  la a1, __data_start__
  la a2, __data_end__
  bgeu a1, a2, 2f
1:
  lw t0, (a0)
  sw t0, (a1)
  addi a0, a0, 4
  addi a1, a1, 4
  bltu a1, a2, 1b
2:

  /* Load tdata section */
  la a0, __tdata_load_start__
  la a1, __tdata_start__
  la a2, __tdata_end__
  bgeu a1, a2, 2f
1:
  lw t0, (a0)
  sw t0, (a1)
  addi a0, a0, 4
  addi a1, a1, 4
  bltu a1, a2, 1b
2:

  /* Load sdata section */
  la a0, __sdata_load_start__
  la a1, __sdata_start__
  la a2, __sdata_end__
  bgeu a1, a2, 2f
1:
  lw t0, (a0)
  sw t0, (a1)
  addi a0, a0, 4
  addi a1, a1, 4
  bltu a1, a2, 1b
2:

  /* Zero bss section */
  la a0, __bss_start__
  la a1, __bss_end__
  bgeu a0, a1, 2f
1:
  sw zero, (a0)
  addi a0, a0, 4
  bltu a0, a1, 1b
2:

  /* Zero tbss section */
  la a0, __tbss_start__
  la a1, __tbss_end__
  bgeu a0, a1, 2f
1:
  sw zero, (a0)
  addi a0, a0, 4
  bltu a0, a1, 1b
2:

  /* Zero sbss section */
  la a0, __sbss_start__
  la a1, __sbss_end__
  bgeu a0, a1, 2f
1:
  sw zero, (a0)
  addi a0, a0, 4
  bltu a0, a1, 1b
2:

  /* Initialize the heap */
  la a0, __heap_start__
  la a1, __heap_end__
  sub a1, a1, a0
  sw zero, 0(a0)
  sw a1, 4(a0)
1:

  /* Call constructors */
  la s0, __ctors_start__
  la s1, __ctors_end__
1:
  beq s0, s1, 2f
  lw s2, 0(s0)
  addi s0, s0, 4
  jalr s2
  j 1b
2:

start:
  li a0, 0
  li a1, 0
  call main

exit:
#ifdef FULL_LIBRARY
  mv s3, a0 // save the exit parameter/return result

  /* Call destructors */
  la s0, __dtors_start__
  la s1, __dtors_end__
1:
  beq s0, s1, 2f
  lw s2, 0(s0)
  addi s0, s0, 4
  jalr s2
  j 1b
2:

  /* Call atexit functions */
  call _execute_at_exit_fns

  /* Call debug_exit with return result/exit parameter */
  mv a0, s3
  tail debug_exit
#endif

  /* Returned from application entry point, loop forever. */
exit_loop:
  j .

.macro HELPER helper_name
  .section .text.\helper_name, "ax", %progbits
  .global \helper_name
#if __riscv_compressed
  .align 1
#else
  .align 2
#endif
  .weak \helper_name
\helper_name:
  .type \helper_name,@function
.endm

HELPER __heap_lock
  ret
HELPER __heap_unlock
  ret
HELPER __printf_lock
  ret
HELPER __printf_unlock
  ret
HELPER __scanf_lock
  ret
HELPER __scanf_unlock
  ret
HELPER __debug_io_lock
  ret
HELPER __debug_io_unlock
  ret
HELPER abort
  j .
HELPER __assert
  j .

HELPER __getchar
  la t0, debug_getchar
  jr t0
HELPER __putchar
  la t0, debug_putchar
  jr t0

/********************************************************************/

        .end

/****** End Of File *************************************************/

